%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Script for creating input files and solving polynomial systems based on
% lamination parameter or ABD term targets. This is essentially a driver
% for the TargConstruction.m and parsePHC.m functions.
%
% - Creates 'square' systems with as many unknown ply angles as targets.
% - Assumes that the PHC executable is in the path
% - Uses the PHC 'blackbox' solver, which appends the full list of
% solutions back to the input file. Might be slow for large systems for
% which it is better to create start systems once and run the coefficient
% continuation step separately.
%
% Updated: 20191003, ADV
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%% GENERAL INPUTS %%%%%%%%%%%%%%%%%%
% The file name for the PHCpack input file:
PHCInputFile = 'ABDEg01_Input.txt';
% The file name for the PHCpack output file:
PHCOutputFile = 'ABDEg01_Output.txt';

% The type of laminate:
LamType = 'asym'; %'reg','sym','asym'

%%%%%%%%%%%%%%% ABD INPUTS %%%%%%%%%%%%%%%%%%%%%%
% ABD TARGETS
% % Some material properties:
h = 0.5;      % Ply thickness (mm)
E1 = 100e3;   % MPa
E2 = 10e3;    % MPa
v12 = 0.25;
G12 = 5e3;    % MPa
props = [E1,E2,v12,G12,h];

% The full matrix of ABD values:
M = nan(6,6);

% Specify the targets, leaving all other entries as NaN:
% A Matrix:
M(1,1) = 140e3; % A11
M(2,2) = 400e3; % A22
% B Matrix
M(1,6) = -55e3; % B13
M(2,6) = -110e3; % B23
% D Matrix
M(4,5) = 250e3; % D12
M(5,5) = 1250e3; % D22

% Construct the input file:
[S,TargStruc] = TargConstruction(M,LamType,PHCInputFile,props);


%%%%%%%%%%%%%%% CALL PHC %%%%%%%%%%%%%%%%%%%%%%%%
% Check if there is already an output file with this name:
if isfile(PHCOutputFile)
     delete(PHCOutputFile)
end

% Run PHC on the input file:
PHCCommand = ['phc -t -b ',PHCInputFile,' ',PHCOutputFile];
% PHCCommand = ['/usr/local/bin/phc -t -b ',PHCInputFile,' ',PHCOutputFile];
[st,re] = system(PHCCommand);

% Parse the result:
[M,solNumVec,varNames,realResults,conjStruc,realAngles] = parsePHC(PHCInputFile);
fprintf('The layups found (in degrees) are: \n')
format short
disp(realAngles)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%





