%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Script for creating input files and solving polynomial systems based on
% lamination parameter or ABD term targets. This is essentially a driver
% for the TargConstruction.m and parsePHC.m functions.
%
% - Creates 'square' systems with as many unknown ply angles as targets.
% - Assumes that the PHC executable is in the path
% - Uses the PHC 'blackbox' solver, which appends the full list of
% solutions back to the input file. Might be slow for large systems for
% which it is better to create start systems once and run the coefficient
% continuation step separately.
%
% Updated: 20191003, ADV
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%% GENERAL INPUTS %%%%%%%%%%%%%%%%%%
% The file name for the PHCpack input file:
PHCInputFile = 'ABDEg02_Input.txt';
% The file name for the PHCpack output file:
PHCOutputFile = 'ABDEg02_Output.txt';

% The type of laminate:
LamType = 'sym'; %'reg','sym','asym'

%%%%%%%%%%%%%%% ABD INPUTS %%%%%%%%%%%%%%%%%%%%%%
% ABD TARGETS
% % Some material properties:
h = 0.5;      % Ply thickness (mm)
E1 = 100e3;   % MPa
E2 = 10e3;    % MPa
v12 = 0.25;
G12 = 5e3;    % MPa
props = [E1,E2,v12,G12,h];

% The full matrix of ABD values:
M = nan(6,6);

% Specify the targets, leaving all other entries as NaN:
% A Matrix:
M(1,1) = 1.5e5; % A11
M(2,2) = 2.0e5; % A22
% D Matrix
M(6,6) = 1e5; % D66

% The linear combination targets:
ABDLinComb = nan(6,6);
% 1st mode buckling problem:
Nx = -1.25e3; % N/mm
m = 1; n = 1; % 1st mode buckling
a = 140; b = 100; % Plate dimensions (mm)
ABDLinComb(4,4) = -pi^2/Nx*(m/a)^2; % D11
ABDLinComb(4,5) = -pi^2/Nx*2/b^2; % D12
ABDLinComb(6,6) = -pi^2/Nx*4/b^2; % D33
ABDLinComb(5,5) = -pi^2/Nx*(a/m)^2*(1/b)^4; % D22

% Construct the input file:
[S,TargStruc] = TargConstruction(M,LamType,PHCInputFile,props,ABDLinComb);


%%%%%%%%%%%%%%% CALL PHC %%%%%%%%%%%%%%%%%%%%%%%%
% Check if there is already an output file with this name:
if isfile(PHCOutputFile)
     delete(PHCOutputFile)
end

% Run PHC on the input file:
PHCCommand = ['phc -t -b ',PHCInputFile,' ',PHCOutputFile];
% PHCCommand = ['/usr/local/bin/phc -t -b ',PHCInputFile,' ',PHCOutputFile];
[st,re] = system(PHCCommand);

% Parse the result:
[M,solNumVec,varNames,realResults,conjStruc,realAngles] = parsePHC(PHCInputFile);
fprintf('The layups found (in degrees) are: \n')
format short
disp(realAngles)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%






