%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% TargConstruction
% Updated: 201901003, ADV
% Description: Constructs a PHCpack input file based on the specified
% combination of laminate targets.
%
% For ABD targets, the M format is a 6x6 matrix M = [A,B;B.',D]
% For lamination parameters, the M format is a 12x1 vector M = [VA;VB;VD]
%
% For ABD targets, it is possible add extra linear combination targets,
% with the coefficients normalised to give a constant term of 1:
% a*Mij + b*Mkl + c*Mmn + ... - 1 = 0
%
% Assumes a ply order of [1,2,...,N] (reverse of some common [.../.../...]
% notations).
% 
% The outputs S and TargStruc contain the 'support' (matrices containing
% the indices present in each equation) and the coefficients of each
% monomial respectively. This may be useful if you wish to export the
% system to another polynomial solver.
%
% Sorts the variables in S and TargStruc into the order:
% [t1,t2,...,u1,u2,...]
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [S,TargStruc] = TargConstruction(M,LamType,PHCInputFile,props,ABDLinComb)
clc
% Set up the original symbols (may not need all of them):
syms t1 t2 t3 t4 t5 t6 t7 t8 t9 t10 t11 t12
syms u1 u2 u3 u4 u5 u6 u7 u8 u9 u10 u11 u12
tiVecFull = [t1,t2,t3,t4,t5,t6,t7,t8,t9,t10,t11,t12];
uiVecFull = [u1,u2,u3,u4,u5,u6,u7,u8,u9,u10,u11,u12];

% Which type of system is being analysed:
if size(M,1) == 6 && size(M,2) == 6
    % A set of ABD targets:
    if exist('ABDLinComb')
        [S,TargStruc] = TargABD(M,LamType,props,tiVecFull,uiVecFull,PHCInputFile,ABDLinComb);
    else
        [S,TargStruc] = TargABD(M,LamType,props,tiVecFull,uiVecFull,PHCInputFile);
    end
elseif size(M,1) == 12 || size(M,2) == 12
    if size(M,1) == 1 || size(M,2) == 1
        % A set of lamination parameter targets:
        [S,TargStruc] = TargLP(M,LamType,tiVecFull,uiVecFull,PHCInputFile);
    else
        error('Unexpected dimensions of input matrix')
    end
    if exist('ABDLinComb')
        error('At this stage linear combination targets only supported for ABD targets (not LP)')
    end
else
    error('Unexpected dimensions of input matrix')
end
end


% Function to write a PHC input file for ABD matrix targets:
function [S,TargStruc] = TargABD(M,LamType,props,tiVecFull,uiVecFull,PHCInputFile,ABDLinComb)
% Initialise the support and target coefficient structures:
S = []; TargStruc = [];
% Extract the material properties:
E1 = props(1);
E2 = props(2);
v12 = props(3);
G12 = props(4);
if length(props)>4
    h = props(5);
end
% Find the primary Q values:
v21 = E2/E1*v12;
Q11 = E1/(1 - v12*v21);
Q22 = E2/(1 - v12*v21);
Q12 = v21*E1/(1 - v12*v21);
Q66 = G12;
% And the invariants
Ua = 1/8*(3*Q11 + 3*Q22 + 2*Q12 + 4*Q66);
Ub = 1/2*(Q11 - Q22);
Uc = 1/8*(Q11 + Q22 - 2*Q12 - 4*Q66);
Ud = 1/8*(Q11 + Q22 + 6*Q12 - 4*Q66);
Ue = 1/8*(Q11 + Q22 - 2*Q12 + 4*Q66);

% How many target entries are there:
n = 0;
% Set up an empty set of target indices:
targIndices = [];
if exist('ABDLinComb')
    m = 0;
    targIndicesLinComb = []; 
end
% Collect the target indices:
for inc = 1:size(M,1)
    for inc2 = 1:size(M,2)
        if ~isnan(M(inc,inc2))
            targIndices(end+1,:) = [inc,inc2];
            n = n+1;
        end
        if exist('ABDLinComb')
            if ~isnan(ABDLinComb(inc,inc2))
                targIndicesLinComb(end+1,:) = [inc,inc2];
                m = m+1;
            end
        end
    end
end
% Check if the ABD targets are a feasible combination:
uiFlag = TargADBCheck(targIndices,LamType);

% Now make sure that the linear comb targets aren't completely contained in
% the other equations:
if exist('ABDLinComb')
    % What are the new terms:
    newLinCombTargs = setdiff(targIndicesLinComb,targIndices,'rows');
    if isempty(newLinCombTargs)
        error('Your ABD linear combination targets are completely defined by your direct ABD targets')
    end
    % Now feed these one at a time through the feasibility check. Need at
    % least one of the new terms to make a feasible set with the other
    % direct targets:
    feasibleFlag = false;
    for inc = 1:size(newLinCombTargs,1)
        try
            uiFlag = TargADBCheck(union(targIndices,newLinCombTargs(inc,:),'rows'),LamType);
            feasibleFlag = true; % If you get to here, at least one must work
        catch
            uiFlag = false;
        end
    end
    % Did any of them work?:
    if ~feasibleFlag
        error('Your selected linear combination targets does not produce an additional linearly dependent equation')
    end
    % Finally, check for any tan terms, and over-write the initial uiFlag:
    tTerms = [1,3;2,3;1,6;2,6;3,4;3,5;4,6;5,6];
    uiFlag = ~(any(ismember(tTerms,union(targIndices,targIndicesLinComb,'rows'),'rows')));
    % and add an extra unknown to the problem:
    n = n + 1;
end

% And the variables:
tiVec = tiVecFull(1:n);
uiVec = uiVecFull(1:n);

%%%%% LAMINATE TYPE %%%%%%
% The mid-plane heights:
if strcmp(LamType,'reg') % REGULAR
    hc = h*((0:n-1)-(n-1)/2);
    N = n;
elseif strcmp(LamType,'sym') % SYMMETRIC
    hc = h*((-n:n-1)+0.5);
    tiVec = [tiVec,fliplr(tiVec)];
    uiVec = [uiVec,fliplr(uiVec)];
    N = 2*n;
elseif strcmp(LamType,'asym') % ANTI-SYMMETRIC
    hc = h*((-n:n-1)+0.5);
    tiVec = [tiVec,-fliplr(tiVec)];
    uiVec = [uiVec,fliplr(uiVec)];
    N = 2*n;
else
    error('Unknown laminate type')
end
%%%%%%%%%%%%%%%%%%%%%%%%%%

% Construct the CLT matrices:
A = zeros(3);
B = zeros(3);
D = zeros(3);
for inc=1:N
    hk = hc(inc) + h/2;    % k surface
    hkm1 =  hc(inc) - h/2; % k-1 surface
    A = A + QxMat(tiVec(inc),uiVec(inc),Ua,Ub,Uc,Ud,Ue)*(hk - hkm1);
    B = B + 1/2*QxMat(tiVec(inc),uiVec(inc),Ua,Ub,Uc,Ud,Ue)*(hk^2 - hkm1^2);
    D = D + 1/3*QxMat(tiVec(inc),uiVec(inc),Ua,Ub,Uc,Ud,Ue)*(hk^3 - hkm1^3);
end
% Assemble into a big matrix:
ABD = [A,B;B.',D];
fileID = fopen(PHCInputFile,'w');
if uiFlag % Check if there were only cos^2 based targets
    fprintf(fileID,'%u\n',n);
else
    fprintf(fileID,'%u\n',2*n);
end
% The equations can now be assembled using the target values:
for inc=1:n
    if exist('ABDLinComb') && inc==n % The extra equation
        % The new equation:
        LinCombEqn = -1;
        for inc2 = 1:m
            LinCombEqn = LinCombEqn + ...
                ABD(targIndicesLinComb(inc2,1),targIndicesLinComb(inc2,2)) ...
                *ABDLinComb(targIndicesLinComb(inc2,1),targIndicesLinComb(inc2,2));
        end
        [thisS,thisCoeffs] = ConstructSupport(LinCombEqn,[tiVecFull(1:n),uiVecFull(1:n)]);
        S(end+1).support = thisS;
        TargStruc(end+1).coeff = thisCoeffs;
        fprintf(fileID,'%s;\n',char(vpa(LinCombEqn)));
    else
        TargEqn = ABD(targIndices(inc,1),targIndices(inc,2))-...
            M(targIndices(inc,1),targIndices(inc,2)); % The usual target equations
        [thisS,thisCoeffs] = ConstructSupport(TargEqn,[tiVecFull(1:n),uiVecFull(1:n)]);
        S(end+1).support = thisS;
        TargStruc(end+1).coeff = thisCoeffs;
        fprintf(fileID,'%s;\n',char(vpa(TargEqn)));
    end
    % If the ti terms are necessary, add the coupling equation:
    if ~uiFlag
        CoupleEqn = uiVec(inc)*(tiVec(inc)^2+1)-1;
        [thisS,thisCoeffs] = ConstructSupport(CoupleEqn,[tiVecFull(1:n),uiVecFull(1:n)]);
        S(end+1).support = thisS;
        TargStruc(end+1).coeff = thisCoeffs;
        fprintf(fileID,'%s;\n',char(vpa(CoupleEqn)));
    end
end
fclose(fileID);
end


% Function to write a PHC input file for lamination parameter targets:
function [S,TargStruc] = TargLP(M,LamType,tiVecFull,uiVecFull,PHCInputFile)
% Initialise the support and target coefficient structures:
S = []; TargStruc = [];
% How many target entries are there:
n = 0;
% Set up an empty set of target indices:
targIndices = [];
% Collect the target indices:
for inc = 1:length(M)
    if ~isnan(M(inc))
        targIndices(end+1) = inc;
        n = n+1;
    end
end
% Check if this is a feasible combination:
uiFlag = TargLPCheck(targIndices,LamType);
% And the variables:
tiVec = tiVecFull(1:n);
uiVec = uiVecFull(1:n);

% Construct a vector of ply mid-plane heights:
h = 1; % w.l.o.g
%%%%% LAMINATE TYPE %%%%%%
% The mid-plane heights:
if strcmp(LamType,'reg') % REGULAR
    hc = h*((0:n-1)-(n-1)/2);
    N = n;
elseif strcmp(LamType,'sym') % SYMMETRIC
    hc = h*((-n:n-1)+0.5);
    tiVec = [tiVec,fliplr(tiVec)];
    uiVec = [uiVec,fliplr(uiVec)];
    N = 2*n;
elseif strcmp(LamType,'asym') % ANTI-SYMMETRIC
    hc = h*((-n:n-1)+0.5);
    tiVec = [tiVec,-fliplr(tiVec)];
    uiVec = [uiVec,fliplr(uiVec)];
    N = 2*n;
else
    error('Unknown laminate type')
end
%%%%%%%%%%%%%%%%%%%%%%%%%%

% Get the thickness of the laminate:
lthick = h*N;
% Construct the lamination parameters:
VA = zeros(4,1);
VB = zeros(4,1);
VD = zeros(4,1);
for inc = 1:N
    hk = hc(inc) + h/2;     % k surface
    hkm1 =  hc(inc) - h/2;  % k-1 surface
    VA = VA + 1/lthick*(hk - hkm1)*trigVec(tiVec(inc),uiVec(inc));
    VB = VB + 2/(lthick^2)*(hk^2 - hkm1^2)*trigVec(tiVec(inc),uiVec(inc));
    VD = VD + 4/(lthick^3)*(hk^3 - hkm1^3)*trigVec(tiVec(inc),uiVec(inc));
end
% Assemble into a big vector:
LP = [VA;VB;VD];
fileID = fopen(PHCInputFile,'w');
if uiFlag % Check if there were only cos^2 based targets
    fprintf(fileID,'%u\n',n);
else
    fprintf(fileID,'%u\n',2*n);
end
% The equations can now be assembled using the target values:
for inc=1:n
    TargEqn = LP(targIndices(inc)) - M(targIndices(inc));
    fprintf(fileID,'%s;\n',char(vpa(TargEqn)));
    [thisS,thisCoeffs] = ConstructSupport(TargEqn,[tiVecFull(1:n),uiVecFull(1:n)]);
    S(end+1).support = thisS;
    TargStruc(end+1).coeff = thisCoeffs;
    % If the ti terms are necessary, add the coupling equation:
    if ~uiFlag
        CoupleEqn = uiVec(inc)*(tiVec(inc)^2+1)-1;
        [thisS,thisCoeffs] = ConstructSupport(CoupleEqn,[tiVecFull(1:n),uiVecFull(1:n)]);
        S(end+1).support = thisS;
        TargStruc(end+1).coeff = thisCoeffs;
        fprintf(fileID,'%s;\n',char(vpa(CoupleEqn)));
    end
end
fclose(fileID);
end


% Function to construct the Qbar matrix
function Q = QxMat(t,u,Ua,Ub,Uc,Ud,Ue)
InvMat = [Ua, 2*u - 1, 8*u^2 - 8*u + 1;
    Ua, 1 - 2*u, 8*u^2 - 8*u + 1;
    Ud, 0, -8*u^2 + 8*u - 1;
    Ue, 0, -8*u^2 + 8*u - 1;
    0, t*u, 4*u*t*(2*u - 1);
    0, t*u, 4*u*t*(1 - 2*u)];
QxVec = InvMat*[1;Ub;Uc];
Q = [QxVec(1, 1), QxVec(3, 1), QxVec(5, 1);
    QxVec(3, 1), QxVec(2, 1), QxVec(6, 1);
    QxVec(5, 1), QxVec(6, 1), QxVec(4, 1)];
end

% Function to construct the vector of trig values for lamination params
function tv = trigVec(t,u)
tv = [2*u-1;2*t*u;8*u^2-8*u+1;4*u*t*(2*u-1)];
% tv = [2*u-1;8*u^2-8*u+1;2*t*u;4*u*t*(2*u-1)]; % Alternative lam param def
end

% A function to check for a feasible combination of inputs for ABDs:
function uiFlag = TargADBCheck(targIndices,LamType)
% Need to do some input verification here:
% (feasible combinations of targets etc.)
% (length of material properties vector)

% There are only certain combinations of targets that are linearly
% independent. Any subset of these combinations should also be ok.
regCombs(1).comb = [1,1;1,2;1,3;2,3];
regCombs(2).comb = [1,1;1,3;2,2;2,3];
regCombs(3).comb = [1,2;1,3;2,2;2,3];
regCombs(4).comb = [1,1;1,3;2,3;3,3];
regCombs(5).comb = [1,3;2,2;2,3;3,3];
asymCombsAD(1).comb = [1,1;1,2];
asymCombsAD(2).comb = [1,1;2,2];
asymCombsAD(3).comb = [1,1;3,3];
asymCombsAD(4).comb = [1,2;2,2];
asymCombsAD(5).comb = [2,2;3,3];
asymCombsB(1).comb = [1,3;2,3];

% Split the targets into separate matrices:
AT = targIndices(ismember(targIndices,[1,1;1,2;1,3;2,2;2,3;3,3],'rows'),:);
BT = targIndices(ismember(targIndices,[1,4;1,5;1,6;2,4;2,5;2,6;3,4;3,5;3,6],'rows'),:);
if ~isempty(BT)
    BT(:,2) = BT(:,2) - 3;
end
DT = targIndices(ismember(targIndices,[4,4;4,5;4,6;5,5;5,6;6,6],'rows'),:);
if ~isempty(DT)
    DT = DT - 3;
end

% First check that only the upper trianglar part of the matrix is used:
if any(targIndices(:,2)-targIndices(:,1) < 0)
    error('Please use only the upper triangular part of the ABD')
end
% Then for symmetry of the B matrix:
if all(ismember([1,2;2,1],BT,'rows')) || ...
        all(ismember([1,3;3,1],BT,'rows')) || ...
        all(ismember([2,3;3,2],BT,'rows'))
    error('B matrix must be symmetric. Do not specify matching off-diagonals')
end
% Flip up any lower-triangular B matrix targets into the top half:
if ismember([2,1],BT,'rows')
    BT(ismember(BT,[2,1],'rows'),:) = [1,2];
elseif ismember([3,1],BT,'rows')
    BT(ismember(BT,[3,1],'rows'),:) = [1,3];
elseif ismember([3,2],BT,'rows')
    BT(ismember(BT,[3,2],'rows'),:) = [2,3];
end

% Initialise the success flags for each sub-matrix:
AFlag = false; BFlag = false; DFlag = false;

if strcmp(LamType,'reg') % REGULAR
    % Make sure that too many targets haven't been specified:
    if size(AT,1)>4 || size(BT,1)>4 || size(DT,1)>4
        error('Limit of four targets per sub-matrix')
    end
    % Check that all the targets are members of at least one feasible set:
    for inc = 1:length(regCombs)
        if all(ismember(AT,regCombs(inc).comb,'rows'))
            AFlag = true;
        end
        if all(ismember(BT,regCombs(inc).comb,'rows'))
            BFlag = true;
        end
        if all(ismember(DT,regCombs(inc).comb,'rows'))
            DFlag = true;
        end
    end
elseif strcmp(LamType,'sym') % SYMMETRIC
    % First check that the B matrix is empty:
    if ~isempty(BT)
        error('No B matrix targets may be specified for symmetric matrices')
    else
        BFlag = true;
    end
    % Make sure that too many targets haven't been specified:
    if size(AT,1)>4 || size(DT,1)>4
        error('Limit of four targets per sub-matrix')
    end
    % Check that all the targets are members of at least one feasible set:
    for inc = 1:length(regCombs)
        if all(ismember(AT,regCombs(inc).comb,'rows'))
            AFlag = true;
        end
        if all(ismember(DT,regCombs(inc).comb,'rows'))
            DFlag = true;
        end
    end
elseif strcmp(LamType,'asym') % ANTI-SYMMETRIC
    % Make sure that too many targets haven't been specified:
    if size(AT,1)>2 || size(BT,1)>2 || size(DT,1)>2
        error('Limit of two targets per sub-matrix')
    end
    % Check that all the targets are members of at least one feasible set:
    for inc = 1:length(asymCombsAD)
        if all(ismember(AT,asymCombsAD(inc).comb,'rows'))
            AFlag = true;
        end
        if all(ismember(DT,asymCombsAD(inc).comb,'rows'))
            DFlag = true;
        end
    end
    for inc = 1:length(asymCombsB)
        if all(ismember(BT,asymCombsB(inc).comb,'rows'))
            BFlag = true;
        end
    end  
end

% See where something went wrong:
if ~AFlag
    error('Unfeasible combination of A matrix targets')
end
if ~BFlag
    error('Unfeasible combination of B matrix targets')
end
if ~DFlag
    error('Unfeasible combination of D matrix targets')
end

% Check if there are no M13 and M23 terms:
tTerms = [1,3;2,3];
uiFlag = ~(any(ismember(tTerms,AT,'rows')) || ...
        any(ismember(tTerms,BT,'rows')) || ...
        any(ismember(tTerms,DT,'rows')));
end


% A function to check for a feasible combination of inputs for LPs:
function uiFlag = TargLPCheck(targIndices,LamType)
% NOTE: All combinations are (theoretically) feasiable for reg type.

if strcmp(LamType,'sym') % SYMMETRIC
    % Make sure there are no VB targets:
    if ~isempty(intersect([5,6,7,8],targIndices))
        error('Symmetric laminates cannot have V_B targets')
    end
elseif strcmp(LamType,'asym') % ANTI-SYMMETRIC
    % Make sure there are no even VA or VD, or odd VB:
    if ~isempty(intersect([2,4,5,7,10,12],targIndices))
        error('Invalid combination of targets for anti-symmetric laminate')
    end
end
% Check if this happens to only contain odd numbered targets.
% This means the polynomial system will not explicitly depend on the ti:
uiFlag = isempty(intersect(2:2:12,targIndices));
end

% A function to extract the support and coefficient list of a given
% symbolic polynomial:
function [thisS,thisCoeffs] = ConstructSupport(myEqn,myVars)
[thisS,thisCoeffs] = SupportScan(myEqn,myVars,1,[],[],[]);
% Check for the origin:
n = size(thisS,2);
CheckVec = thisS * ones(n,1);
if isempty(find(CheckVec==0)) == 1  %#ok<EFIND>
    thisS = [zeros(1,n);thisS];
    thisCoeffs = [0;thisCoeffs];
end
end

% The recursion function for scanning through all the polynomial terms:
function [thisS,thisCoeffs] = SupportScan(thisTerm,myVars,varNum,powerList,thisS,thisCoeffs)
% Perform the intial coefficient scan:
coeffList = coeffs(thisTerm,myVars(varNum),'all');
% and reverse it to get the powers in the right order:
coeffList = fliplr(coeffList);
% Loop over the coefficient list:
for inc = 1:length(coeffList)
    % Check if we're already at the last variable:
    if length(myVars) == varNum
        % Make sure it's not a zero coefficient:
        if abs(coeffList(inc)) > 1e-12
            thisS = [thisS;[powerList,inc-1]];
            thisCoeffs = [thisCoeffs;double(coeffList(inc))];
        end
    else
        % Otherwise go round again:
        [thisS,thisCoeffs] = SupportScan(coeffList(inc),myVars,varNum+1,[powerList,inc-1],thisS,thisCoeffs);
    end
end

end

