% A function to read in a PHC input file with results appended.
% Not for the output file from -b itself
% Solutions are stored in the output matrix from the function.
%
% Note that it is assumed that the variable names will be ti for tan and ui
% for cos^2.
%
% Sorts the variables into the order [t1;t2;...;u1;u2;...]
%
% Updated 20190924, ADV
%
function [M,solNumVec,varNames,realResults,conjStruc,realAngles] = parsePHC(fileName)
% Create an empty solution array:
M = [];
% and an empty corresponding solution number vector:
solNumVec = [];
% and an empty variable name cell array:
varNames = {};
% Set the line counter:
lineCount = 0;
% and a local vector to store each solution:
thisSolVec = [];
% Open the file
fid = fopen(fileName);
while ~feof(fid)
    % Get the line
    textLineEntry = fgetl(fid);
    % Check if the first word is 'solution':
    if length(textLineEntry) >= 8 && contains(textLineEntry(1:8),'solution')
        % Find the location of the first colon:
        colonNum = strfind(textLineEntry,':');
        % Find the solution number:
        solNum = textscan(textLineEntry(10:colonNum(1)-1),'%f');
        solNum = solNum{1};
        solNumVec = [solNumVec,solNum];
    end
    % Next look for the start of the solutions, which begins under a line
    % starting 'the solution':
    if length(textLineEntry) >= 12 && contains(textLineEntry(1:12),'the solution')
        % Start the line counter:
        lineCount = 1;
        % and start the loop again:
        continue
    end
    % Check if this is the end of a solution:
    if length(textLineEntry) >= 2 && contains(textLineEntry(1:2),'==')
        % Store the solution:
        M = [M,thisSolVec];
        % and reset:
        lineCount = 0;
        thisSolVec = [];
    end
    % Read in the data:
    if lineCount >= 1
        % Find the colon:
        colonNum = strfind(textLineEntry,':');
        % Read in the variable names if this is the first solution:
        if isempty(M)
            varNames{lineCount,1} = strtrim(textLineEntry(1:colonNum(1)-1));
        end
        % Scan in both parts of the solution:
        solScan = textscan(textLineEntry(colonNum(1)+1:end),'%f %f');
        % Convert to a complex number:
        thisSolVec = [thisSolVec;solScan{1}+1i*solScan{2}];
        % Increment the counter:
        lineCount = lineCount + 1;
    end
    
end
fclose(fid);

% Categorise the results:
[realResults,conjStruc,realAngles,varNames] = CategoriseResults(M,varNames);
end


% A simple script to categorise the output solutions.
function [realResults,conjStruc,realAngles,varNames] = CategoriseResults(M,varNames)
% First make sure the results are in the following sequence:
% t1
% t2
% .
% .
% u1
% u2
% .
% .
% First check if there are any tan results:
tanIndices = find(contains(varNames,'t'));
% and the same for the cos^2 terms:
uIndices = find(contains(varNames,'u'));
% Move the results matrix into a new variable:
M2 = M; varNames2 = varNames;
% and move the rows back into the desired order:
for inc = tanIndices'
    M(str2num(varNames2{inc}(2:end)),:) = M2(inc,:);
    varNames{str2num(varNames2{inc}(2:end))} = varNames2{inc};
end
for inc = uIndices'
    M(str2num(varNames2{inc}(2:end))+length(tanIndices),:) = M2(inc,:);
    varNames{str2num(varNames2{inc}(2:end))+length(tanIndices)} = varNames2{inc};
end

myTol = 1e-5;

% Total number of results:
fprintf('The total number of solutions is: %d\n',size(M,2))

% Remove the NaN results:
nanIncs = [];
for inc=1:size(M,2)
    thisX = M(:,inc);
    if isnan(thisX(1))
        nanIncs(1,end+1) = inc;
    end
end
% Remove them:
M(:,nanIncs) = [];
fprintf('The number of NaN solutions is: %d\n',size(nanIncs,2))

% Find the real ones:
realResults = [];
realIncs = [];
for inc=1:size(M,2)
    if norm(imag(M(:,inc)))<myTol
        realResults(:,end+1) = real(M(:,inc));
        realIncs(1,end+1) = inc;
    end
end
% Remove them:
M(:,realIncs) = [];
fprintf('The number of real solutions is: %d\n',size(realIncs,2))

% Finally, find the conjugate pairs:
sameIncs = [];
conjIncs = [];
inc = 1;
while size(M,2)>0
    thisX = M(:,1);
    % Check to see if this same solution has popped up already:
    sameIncs = find(sum(abs(M-repmat(thisX,1,size(M,2))),1)<myTol);
    % and now look for the conjugate:
    conjIncs = find(sum(abs(M-repmat(conj(thisX),1,size(M,2))),1)<myTol);
    % Add them to a structure:
    conjStruc(inc).pairs = M(:,sameIncs);
    conjStruc(inc).pairs = [conjStruc(inc).pairs,M(:,conjIncs)];
    % And remove:
    M(:,[sameIncs,conjIncs]) = [];
    inc = inc+1;
end
if exist('conjStruc')
    fprintf('The number of imaginary conjugate pairs is: %d\n',length(conjStruc))
else
    conjStruc = [];
end

% If there were any real results, let's turn them into angles:
if ~isempty(realResults)
    % Check if there are any tan results:
    tanIndices = find(contains(varNames,'t'));
    if isempty(tanIndices)
        % Must be only cos^2 terms (no sign information)
        fprintf('Note: only cos^2 terms found to be present. No angle sign information is available.\n')
        realAngles = acosd(sqrt(realResults));
    else
        realAngles = atand(realResults(tanIndices,:));
    end
else
    realAngles = [];
end
end